import UIKit
import CoreData

class ContactsTableViewController : UITableViewController, NSFetchedResultsControllerDelegate {
    
    var fetchedResultsController: NSFetchedResultsController<NSFetchRequestResult>!
    var names : Array<String>!
    var surnames : Array<String>!

    
override func viewDidLoad() {
        super.viewDidLoad()

        // Uncomment the following line to preserve selection between presentations
        // self.clearsSelectionOnViewWillAppear = false

        // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
        self.navigationItem.leftBarButtonItem = self.editButtonItem
        
        initializeFetchedResultsController()
        
        if let path = Bundle.main.path(forResource: "names", ofType:"csv"){
            do {
                let data = try String(contentsOfFile: path, encoding:.utf8)
                self.names = data.components(separatedBy: .newlines)
                
            } catch {
                print(error)
            }
        }
        
        if let path = Bundle.main.path(forResource: "surnames", ofType:"csv"){
            do {
                let data = try String(contentsOfFile: path, encoding:.utf8)
                self.surnames = data.components(separatedBy: .newlines)
                
            } catch {
                print(error)
            }
        }
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    // MARK: - Table view data source

    override func numberOfSections(in tableView: UITableView) -> Int {
        // #warning Incomplete implementation, return the number of sections
        return fetchedResultsController.sections!.count
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        // #warning Incomplete implementation, return the number of rows
        
        guard let sections = fetchedResultsController.sections else {
            fatalError("No sections in fetchedResultsController")
        }
        
        let sectionInfo = sections[section]
        return sectionInfo.numberOfObjects
    }
    
    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        
        let cell = tableView.dequeueReusableCell(withIdentifier: "ContactCell", for: indexPath)

        // Configure the cell...
        let object = self.fetchedResultsController.object(at: indexPath) as! Contact
        let name = object.name
        let surname = object.surname
        cell.textLabel?.text = name! + " " + surname!
        return cell
    }
    
    // Override to support editing the table view.
    override func tableView(_ tableView: UITableView, commit editingStyle: UITableViewCellEditingStyle, forRowAt indexPath: IndexPath) {
        if editingStyle == .delete {
            // Delete the row from the data source
            let appDelegate = UIApplication.shared.delegate as! AppDelegate
            let moc = appDelegate.dataController.persistentContainer.viewContext
            let object = self.fetchedResultsController.object(at: indexPath) as! Contact
            moc.delete(object)
            
            do {
                try moc.save()
                tableView.deleteRows(at: [indexPath], with: .fade)
            } catch {
                print("Could not save delete")
            }
        } else if editingStyle == .insert {
            // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
        }    
    }
    
    func initializeFetchedResultsController() {
        let request = NSFetchRequest<NSFetchRequestResult>(entityName: "Contact")
        let lastNameSort = NSSortDescriptor(key: "surname", ascending: true)
        request.sortDescriptors = [lastNameSort]
    
        let appDelegate = UIApplication.shared.delegate as! AppDelegate
        let moc = appDelegate.dataController.persistentContainer.viewContext
        
        self.fetchedResultsController = NSFetchedResultsController(fetchRequest: request, managedObjectContext: moc, sectionNameKeyPath: nil, cacheName: nil)
        self.fetchedResultsController.delegate = self
        
        do {
            try fetchedResultsController.performFetch()
        } catch {
            fatalError("Failed to initialize FetchedResultsController: \(error)")
        }
    }
    
    func controller(_ controller: NSFetchedResultsController<NSFetchRequestResult>, didChange anObject: Any, at indexPath: IndexPath?, for type: NSFetchedResultsChangeType, newIndexPath: IndexPath?) {
        switch type {
        case .insert:
            tableView.insertRows(at: [newIndexPath!], with: .fade)
        default:
            break
        }
    }
    
    @IBAction func addContact(_ sender: Any) {
        let appDelegate = UIApplication.shared.delegate as! AppDelegate
        let moc = appDelegate.dataController.persistentContainer.viewContext
        let person = NSEntityDescription.insertNewObject(forEntityName: "Contact", into: moc) as! Contact
        
        let randomIndex = Int(arc4random_uniform(UInt32(self.names.count)))
        let randomSurnameIndex = Int(arc4random_uniform(UInt32(self.surnames.count)))
        
        person.name = self.names[randomIndex]
        person.surname = self.surnames[randomSurnameIndex]
        
        do {
            try moc.save()
        } catch {
            fatalError("Failure to save context \(error)")
        }
    }

}
